<?php

namespace Drupal\feeds_plupload\Feeds\Fetcher\Form;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Form\FormStateInterface;
use Drupal\feeds\FeedInterface;
use Drupal\feeds\Plugin\Type\ExternalPluginFormBase;

/**
 * Provides a form on the feed edit page for the plupload fetcher.
 */
class PluploadFetcherFeedForm extends ExternalPluginFormBase {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(
    array $form,
    FormStateInterface $form_state,
    FeedInterface $feed = NULL
  ) {
    $form['source'] = [
      '#type' => 'plupload',
      '#title' => t('Upload files'),
      '#required' => TRUE,
      '#autoupload' => $this->plugin->getConfiguration('autoupload'),
      '#autosubmit' => $this->plugin->getConfiguration('autosumbit'),
      # Note: This form overrides the upsteram #element_validate callback
      # completely in order to avoid a call to File::create. Note also that
      # #upload_validators is only specified in order to supply the
      # allowed_extension config to PlUploadFile::valueCallback.
      '#element_validate' => [
        [get_class($this), 'validatePlUploadFile'],
      ],
      '#upload_validators' => [
        'file_validate_extensions' => [
          $this->plugin->getConfiguration('allowed_extensions'),
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(
    array &$form,
    FormStateInterface $form_state,
    FeedInterface $feed = NULL
  ) {
    $feed->setSource(Json::encode($form_state->getValue('source')));
  }

  /**
   * Render API callback: Validates the managed_file element.
   *
   * @see \Drupal\plupload\Element\PlUploadFile::validatePlUploadFile()
   * @see file_validate_extensions
   */
  public static function validatePlUploadFile(&$element, FormStateInterface $form_state, &$complete_form) {
    $extensions = $element['#upload_validators']['file_validate_extensions'][0];
    $regex = '/\.(' . preg_replace('/ +/', '|', preg_quote($extensions)) . ')$/i';

    foreach ($element['#value'] as $file_info) {
      if (!preg_match($regex, $file_info['name'])) {
        $message = t('Only files with the following extensions are allowed: %files-allowed.', ['%files-allowed' => $extensions]);
        $form_state->setError($element, $message);
      }
    }
  }

}
